@@ -2,23 +2,22 @@
 #
 # Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
-"""Generic expat-based XML parser base class.
-
-Modified for Roundup 0.5 release:
-
-- removed dependency on zLOG
-
+# Modifications for Roundup:
+# 1. commented out zLOG references
 """
-__docformat__ = 'restructuredtext'
+Generic expat-based XML parser base class.
+"""
+
+#import zLOG
 
 class XMLParser:
 
