@@ -16,8 +16,6 @@
 #
 
 """Command-line script that runs a server over roundup.cgi.client.
-
-$Id: roundup_server.py,v 1.94 2007-09-25 04:27:12 jpend Exp $
 """
 __docformat__ = 'restructuredtext'
 
@@ -362,6 +360,9 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         if os.environ.has_key('CGI_SHOW_TIMING'):
             env['CGI_SHOW_TIMING'] = os.environ['CGI_SHOW_TIMING']
         env['HTTP_ACCEPT_LANGUAGE'] = self.headers.get('accept-language')
+        range = self.headers.getheader('range')
+        if range:
+            env['HTTP_RANGE'] = range
 
         # do the roundup thing
         tracker = self.get_tracker(tracker_name)
@@ -462,9 +463,16 @@ class ServerConfig(configuration.Config):
 
     SETTINGS = (
             ("main", (
-            (configuration.Option, "host", "",
+            (configuration.Option, "host", "localhost",
                 "Host name of the Roundup web server instance.\n"
-                "If empty, listen on all network interfaces."),
+                "If left unconfigured (no 'host' setting) the default\n"
+                "will be used.\n"
+                "If empty, listen on all network interfaces.\n"
+                "If you want to explicitly listen on all\n"
+                "network interfaces, the address 0.0.0.0 is a more\n"
+                "explicit way to achieve this, the use of an empty\n"
+                "string for this purpose is deprecated and will go away\n"
+                "in a future release."),
             (configuration.IntegerNumberOption, "port", DEFAULT_PORT,
                 "Port to listen on."),
             (configuration.NullableFilePathOption, "favicon", "favicon.ico",
@@ -731,7 +739,10 @@ Options:
  -h            print this text and exit
  -S            create or update configuration file and exit
  -C <fname>    use configuration file <fname>
- -n <name>     set the host name of the Roundup web server instance
+ -n <name>     set the host name of the Roundup web server instance,
+               specifies on which network interfaces to listen for
+               connections, defaults to localhost, use 0.0.0.0 to bind
+               to all network interfaces
  -p <port>     set the port to listen on (default: %(port)s)
  -l <fname>    log to the file indicated by fname instead of stderr/stdout
  -N            log client machine names instead of IP addresses (much slower)
