@@ -21,6 +21,10 @@ class RFC2822TestCase(unittest.TestCase):
         result = '=?utf-8?q?Re:_[it=5Fissue3]_Ren=C3=A9s_[status=3Dfeedback]?='
         self.assertEqual(encode_header(src), result)
 
+        src = 'Was machen\xc3\xbc und Fragezeichen?'
+        result = '=?utf-8?q?Was_machen=C3=BC_und_Fragezeichen=3F?='
+        self.assertEqual(encode_header(src), result)
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(RFC2822TestCase))
