@@ -146,12 +146,14 @@ class HTMLClassTestCase(TemplatingTestCase) :
     def test_url_replace(self):
         p = StringHTMLProperty(self.client, 'test', '1', None, 'test', '')
         def t(s): return p.hyper_re.sub(p._hyper_repl, s)
-        ae = self.assertEquals
+        ae = self.assertEqual
         ae(t('item123123123123'), 'item123123123123')
         ae(t('http://roundup.net/'),
            '<a href="http://roundup.net/">http://roundup.net/</a>')
         ae(t('&lt;HTTP://roundup.net/&gt;'),
            '&lt;<a href="HTTP://roundup.net/">HTTP://roundup.net/</a>&gt;')
+        ae(t('&lt;http://roundup.net/&gt;.'),
+            '&lt;<a href="http://roundup.net/">http://roundup.net/</a>&gt;.')
         ae(t('&lt;www.roundup.net&gt;'),
            '&lt;<a href="http://www.roundup.net">www.roundup.net</a>&gt;')
         ae(t('(www.roundup.net)'),
@@ -165,6 +167,12 @@ class HTMLClassTestCase(TemplatingTestCase) :
         ae(t('(e.g. http://en.wikipedia.org/wiki/Python_(programming_language)).'),
            '(e.g. <a href="http://en.wikipedia.org/wiki/Python_(programming_language)">'
            'http://en.wikipedia.org/wiki/Python_(programming_language)</a>).')
+        ae(t('(e.g. http://en.wikipedia.org/wiki/Python_(programming_language))&gt;.'),
+           '(e.g. <a href="http://en.wikipedia.org/wiki/Python_(programming_language)">'
+           'http://en.wikipedia.org/wiki/Python_(programming_language)</a>)&gt;.')
+        ae(t('(e.g. http://en.wikipedia.org/wiki/Python_(programming_language&gt;)).'),
+           '(e.g. <a href="http://en.wikipedia.org/wiki/Python_(programming_language">'
+           'http://en.wikipedia.org/wiki/Python_(programming_language</a>&gt;)).')
 
 '''
 class HTMLPermissions:
